import json
from pathlib import Path
from datetime import datetime
from termcolor import colored, cprint

_CURDIRT = Path(__file__).parent
_TASK_FILE = _CURDIRT / 'tasks.json'

"""
Structure of the json file:
{
    'todo':[
        {
            'name': str,
            'subject': str,
            'deadline': str
        },
        ...
    ],
    'review': [
        ...
    ]
}
"""

def _print_list(task_list : list):
    """
    Print a list of task in column
    Parameters:
        task_list: a list of task objects
    """
    for i in range(len(task_list)):
        item = task_list[i]
        name = item['name']
        subject = item['subject']
        deadline = item['deadline']
        log_string = f'{i+1}. {name}({subject}) - {deadline}'
        date_obj = datetime.strptime(deadline, '%Y-%m-%d').date()
        if date_obj == datetime.today().date():
            print(colored(log_string, 'red', attrs=['bold']))
        else:
            print(log_string)

    print("")

def read_tasks_and_print():
    """
    Read and print all tasks from the local
    json file created.
    """
    if not _TASK_FILE.is_file():
        print("No existing tasks! Try to add some.")
        return
    
    with open(_TASK_FILE) as f:
        task_dict = json.load(f)
        todo_list = task_dict['todo']
        review_list = task_dict['review']

        cprint("Todos:", 'grey', 'on_yellow', end='\n')
        _print_list(todo_list)
        
        cprint("Reviews:", 'white', 'on_blue', end='\n')
        _print_list(review_list)

def _add_task(name: str, deadline: str, subject: str):
    """
    Add a new task to the json file
    Parameter:
        name: title of the task
        deadline: deadline of the task
        subject: which course this assignment belongs to
    """
    data = {
        "todo": [],
        "review": []
    }
    if _TASK_FILE.is_file():
        with open(_TASK_FILE) as json_file:
            data = json.load(json_file)
    
    task = {}
    task['name'] = name
    task['subject'] = subject
    task['deadline'] = deadline
    data['todo'].append(task)

    with open(_TASK_FILE, 'w') as json_file:
        json.dump(data, json_file)

def add_task_from_input():
    """
    Use console input prompts to get user data
    for adding a new task
    """
    name = input("Title of the task: ")
    subject = input("Which course? (default: Misc): ")
    if subject == "" or subject is None:
        subject = "Misc"
    
    valid_date = False

    while not valid_date:
        deadline = input("When should you finish it: ")
        try:
            datetime.strptime(deadline, "%Y-%m-%d")
            valid_date = True
        except ValueError:
            print("Invalid deadline format: please use YYYY-MM-DD format!")

    _add_task(name, deadline, subject)

    print("Task added!")

def complete_task(index: int):
    """
    Complete a task and move it from Todo list
    to Review list

    Parameters:
        index: the index of the task in the todo list
    """
    if not _TASK_FILE.is_file():
        print("You don't have a task yet.")
    else:
        data = {}
        with open(_TASK_FILE) as json_file:
            data = json.load(json_file)
        todo_list : list = data['todo']
        if len(todo_list) >= index:
            data['review'].append(todo_list.pop(index-1))
            with open(_TASK_FILE, 'w') as json_file:
                json.dump(data, json_file)
            print("Task completed and wait for review later.")
        else:
            print("You typed a wrong task index")

def _terminate_task(type: str, index: int):
    """
    Terminate a task.

    Parameters:
        type: the bucket of the task
        index: which task inside the bucket
    """
    if not _TASK_FILE.is_file():
        print("You don't have a task yet.")
    else:
        data = {}
        with open(_TASK_FILE) as json_file:
            data = json.load(json_file)

        if type in data:
            task_list = data[type]
            if len(task_list) >= index:
                task_list.pop(index-1)
                with open(_TASK_FILE, 'w') as json_file:
                    json.dump(data, json_file)
                print("Task closed.")
            else:
                print("You typed a wrong task index")
        else:
            print("You can only close task from either TODO or REVIEW bucket.")

def _validate_index_string(index_string: str) -> bool:
    """
    Handle validation of index string from the argument. 
    a string should be in format 't{%d}' or 'r{%d}'
    
    Parameters:
        index_string: the string we want to validate
    Returns:
        a boolean indicating whether the index string is valid
    """
    if len(index_string) < 2:
        print("Invalid index string length!")
        return False
    elif index_string[0] != 't' and index_string[0] != 'r':
        print("Invalid index string prefix!")
        return False
    elif not index_string[1:].isnumeric():
        print("Index need to have a number suffix!")
        return False
    else:
        return True

def terminate_task_by_string(index_string: str):
    """
    Terminate a task with a index string
    """
    if _validate_index_string(index_string):
        list_type = 'todo' if index_string[0] == 't' else 'review'
        index = int(index_string[1:])
        _terminate_task(list_type, index)


import requests
from bs4 import BeautifulSoup
from typing import Dict
from html import unescape
from termcolor import colored
from datetime import datetime
from unidecode import unidecode
from enum import Enum

MENU_URL = "http://menu.dining.ucla.edu/Menus"
HOUR_URL = "http://menu.dining.ucla.edu/Hours"

class Period(Enum):
    """
    Enum for different meal periods
    """
    BREAKFAST = "Breakfast"
    LUNCH = "Lunch"
    DINNER = "Dinner"

    @classmethod
    def contains(cls, input: str) -> bool:
        """
        Check whether a string is one of the period.
        
        Parameters:
            input: the string we want to check
        
        Returns:
            a boolean indicating whether the input is a Period
        """
        try:
            cls(input)
        except:
            return False
        return True

def _get_html(url: str) -> str:
    """
    Return HTML raw text from a URL

    Parameters:
        url: the url to the website we want to fetch
    Returns:
        a raw text of the website
    Raise:
        exception when there is a network error.
    """
    request = requests.get(url)
    if request.status_code == 200:
        return request.content
    else:
        raise Exception("Network Error! Check your connection.")

def _format_output(html: str) -> Dict[str, Dict[str, list]]:
    """
    Extract information from the menu website and parse it
    into the dict for printing

    Parameters:
        html: the raw text of the website
    Return:
        dict
    """
    soup = BeautifulSoup(html, "lxml")
    res_dict = {}

    times = soup.find_all('h2')
    menu_blocks = soup.find_all('div', {'class': "menu-block whole-col"})
    meal_dict = dict(zip(times, menu_blocks))
    for time, menu_block in meal_dict.items():
        period = time.text
        res_dict[period] = {}
        restaurants = menu_block.find_all('h3')
        menu = menu_block.find_all("ul", {"class": "sect-list"})
        menu_dict = dict(zip(restaurants, menu))

        for key, value in menu_dict.items():
            name = key.text
            res_dict[period][name] = []
            dishes = value.find_all('a', {"class": "recipelink"})
            for link in dishes:
                res_dict[period][name].append(unescape(link.text))
    
    return res_dict

def _print_menu(menu_dict: Dict[str, Dict[str, list]]) -> None:
    """
    Print the menu dictionary
    """
    for key, value in menu_dict.items():
        print(colored(key+":\n", "blue", attrs=['bold']))
        for dining_hall, dishes in value.items():
            print(colored(dining_hall+":", "white", attrs=['bold']))
            for dish in dishes:
                print("\t"+dish)
            print("")

def print_menu_all() -> None:
    """
    Print Overview menu
    """
    _print_menu(_format_output(_get_html(MENU_URL)))

def print_menu_detail_all(type: Period) -> None:
    """
    Print detail menu for a specific period
    Parameter:
        type: a meal Period
    """
    _print_menu(_format_output(_get_html(MENU_URL+"/"+type)))

def _get_hour_info_all() -> Dict[str, Dict[str, str]]:
    """
    Fetch hour of operation info for each dining hall
    Return:
        a dictionary from dining hall to hour of operation
    """
    url = HOUR_URL + "/" + datetime.now().date().strftime("%Y-%m-%d")
    html = _get_html(url)
    soup = BeautifulSoup(html, 'lxml')
    res_dict = {}

    table = soup.find('table', {'class': 'hours-table'})
    table_heads = table.find('thead').find('tr').find_all('th', {'class': 'hours-head'})
    table_body = table.find('tbody')
    rows = table_body.find_all('tr')

    period = []
    for header in table_heads:
        content = unidecode(header.text).strip()
        if content != "":
            period.append(header.text)
    
    for row in rows:
        cols = row.find_all('td')
        dining_hall = cols[0].find('span').text
        ranges = []
        for i in range(1, len(cols)):
            if cols[i].find('span'):
                ranges.append(cols[i].find('span').text.strip(' \r\n'))
            else:
                ranges.append(cols[i].text.strip(' \r\n'))
        
        res_dict[dining_hall] = dict(zip(period, ranges))
    
    return res_dict

def print_hour_all() -> None:
    """
    Print hour of operation for every dining hall
    """
    hour_dict = _get_hour_info_all()
    date_string = datetime.now().date().strftime("%B %d, %Y")
    print(colored(date_string+"\n", 'cyan', attrs=['underline']))
    for key, value in hour_dict.items():
        print(colored(key+":", 'white', attrs=['bold']))
        for period, range in value.items():
            print('\t', period, ': ', range)
        print("")

def print_hour(dining_hall: str) -> None:
    """
    Print hour of operation for one dining hall
    Parameters:
        dining_hall: the name of dining hall
    """
    hour_dict = _get_hour_info_all()
    date_string = datetime.now().date().strftime("%B %d, %Y")
    if dining_hall in hour_dict:
        print(colored(date_string+"\n", 'cyan', attrs=['underline']))
        print(colored(dining_hall+":", 'white', attrs=['bold']))
        for period, range in hour_dict[dining_hall].items():
            print('\t', period, ': ', range)
        print("")
    elif dining_hall == 'all' or dining_hall == '':
        print_hour_all()
    else:
        print(dining_hall, "is not opening today :(")


    




import argparse
from bruin.meal import print_menu_all, print_hour, print_menu_detail_all, Period
from bruin.calendar_tool import print_events_today
from bruin.tasks import add_task_from_input, read_tasks_and_print, terminate_task_by_string, complete_task

parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter)

parser.add_argument("option", type=str, help=
    "Tools that can be used in this cli, including:\n\n\
meal: print today's menu for each dining hall.\n \
calendar: print incoming events/classes today.\n \
tasks: Add and organize your daily tasks."
)

parser.add_argument(
    "--hour", 
    type=str, 
    dest='hour', 
    action='store', 
    default=None, 
    help='Used for meal: fetch hour of operation for dining halls [=all, =\'De Neve\', etc.]'
)

parser.add_argument(
    "-d",
    "--detail", 
    type=str, 
    dest='detail', 
    action='store', 
    default=None, 
    help='Print detail menu for [=Breakfast, =Lunch, =Dinner]'
)

parser.add_argument(
    "-a",
    "--add",
    dest='add',
    action='store_true',
    default=False,
    help="Used for tasks: add a new todo."
)

parser.add_argument(
    "-c",
    "--complete",
    dest='complete',
    action='store',
    type=int,
    default=None,
    help="Used for tasks: complete a task and move to review bucket."
)

parser.add_argument(
    "-t",
    "--terminate",
    dest='terminate',
    action='store',
    type=str,
    default=None,
    help='Used for tasks: remove a task from the list.'
)

def main():
    args = parser.parse_args()
    if args.option == "meal":
        if args.hour is not None:
            print_hour(args.hour)
        elif args.detail is not None:
            if Period.contains(args.detail):
                print_menu_detail_all(args.detail)
            else:
                print("Invalid argument for --detail:")
                parser.print_help()
        else:
            print_menu_all()
    elif args.option == "calendar":
        print("Reminder: Please import your calendar data into your Google Calendar!\n")
        print_events_today()
    elif args.option == "tasks":
        if args.add:
            add_task_from_input()
        elif args.complete is not None:
            complete_task(args.complete)
        elif args.terminate is not None:
            terminate_task_by_string(args.terminate)
        else:
            read_tasks_and_print()
    else:
        parser.print_help()

if __name__ == '__main__':
    main()


from datetime import datetime, timedelta, timezone
import os
from googleapiclient.discovery import build
from google_auth_oauthlib.flow import InstalledAppFlow
from google.auth.transport.requests import Request
from google.oauth2.credentials import Credentials
from iso8601 import parse_date
from termcolor import colored
from pathlib import Path

CURDIRT = Path(__file__).parent
SCOPES = ['https://www.googleapis.com/auth/calendar.readonly']
_GOOGLE_SERVICE = None

def _init_google_service():
    """
    Initialize google API client with OAuth 2.
    """
    global _GOOGLE_SERVICE
    if _GOOGLE_SERVICE is not None:
        return

    creds = None
    path_to_secret = os.path.join(CURDIRT, "secret/client_secret.json")
    path_to_token = os.path.join(CURDIRT, "secret/token.json")
    if os.path.exists(path_to_token):
        creds = Credentials.from_authorized_user_file(path_to_token, scopes=SCOPES)
    
    if not creds or not creds.valid:
        if creds and creds.expired and creds.refresh_token:
            creds.refresh(Request())
        else:
            flow = InstalledAppFlow.from_client_secrets_file(path_to_secret, scopes=SCOPES)
            creds = flow.run_local_server(port=0)
        with open(path_to_token, 'w') as f:
            f.write(creds.to_json())
    
    _GOOGLE_SERVICE = build('calendar', 'v3', credentials=creds)


def print_events_today():
    """
    Fetch calendar events in 24 hours for google calendar
    """
    _init_google_service()
    now = datetime.utcnow().isoformat() + 'Z' # 'Z' indicates UTC time
    later = (datetime.utcnow() + timedelta(days=1)).isoformat() + 'Z'
    print('Getting today\'s events...\n')
    events_result = _GOOGLE_SERVICE.events().list(
        calendarId='primary', 
        timeMin=now,
        timeMax=later,
        singleEvents=True,
        orderBy='startTime'
    ).execute()
    events = events_result.get('items', [])

    if not events:
        print('No upcoming events found.')
    for event in events:
        start = event['start'].get('dateTime', None)
        reminder = False
        if start is None:
            start = event['start'].get('date')
        else:
            start = parse_date(start)
            difference = (start - datetime.now(timezone.utc)).total_seconds()
            mins = divmod(difference, 60)[0]
            start = start.strftime("%Y-%m-%d, %H:%M:%S")
            if mins < 15 and mins >= 0:
                reminder = True

        if reminder:
            print(colored(start + " "+ event['summary'] + "\t <-- incoming", 'cyan', attrs=['bold']))
        else:
            print(start, event['summary'])



from setuptools import setup, find_packages
from io import open
from os import path
import pathlib

CURDIRT = pathlib.Path(__file__).parent

README = (CURDIRT / "README.md").read_text()

with open(path.join(CURDIRT, 'requirements.txt'), encoding='utf-8') as f:
    all_reqs = f.read().split('\n')

install_requires = [x.strip() for x in all_reqs if ('git+' not in x) and (not x.startswith("#"))]
dependency_links = [x.strip().replace('git+', '') for x in all_reqs if 'git+' not in x]

setup(
    name='bruin',
    description="Command line tool made by python for various toolbox used in UCLA",
    version="0.3.1",
    packages=find_packages(),
    install_requires=install_requires,
    python_requires='>=3.6',
    entry_points='''
        [console_scripts]
        bruin=bruin.__main__:main
    ''',
    include_package_data=True,
    dependency_links=dependency_links,
    license='MIT',
    author='Oswald He',
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/OswaldHe/bruin-cli"
)